from setuptools import setup, find_packages

setup(name="ZeroMqServiceLocator",
      version="2.0.0",
      packages=find_packages(where="."),
      py_modules=["main"],
      install_requires=['ZeroMqMessageHandler==2.0.0', 'EdpHelper==0.15', 'MessageProcessor==3.0.2'],
      url="http://www.e-deploy.com.br",
      author="Eduardo Almeida",
      author_email="eduardo.almeida@e-deploy.com.br")
